import time
import uuid
from queue import Queue
from threading import Thread

from yodu.db.es.es_client import ESClient
from yodu.provider.provider_base import ProviderBase
from yodu.recommeder.db.influx_db import InfluxDb

NUM_THREADS = 5
que = Queue()
threads_list = list()


class RecommenderEngine:
    __recommender_id = None
    __providers = None
    __es_client = None
    __influxdb_client = None
    __algo_spec = None

    def __init__(self, recommender_id, algo_spec):
        if not recommender_id:
            self.recommender_id = recommender_id
        else:
            self.recommender_id = uuid.uuid4()
        self.__es_client = ESClient()
        self.__influxdb_client = InfluxDb().get_client()
        self.__algo_spec = algo_spec
        self.set_algo_specifications(algo_spec)

    def set_algo_specifications(self, algo_spec):
        for name, config in algo_spec.items():
            self.add_provider(name, config["provider"])

    def add_provider(self, name: str, provider: ProviderBase):
        if not self.__providers:
            self.__providers = {}
        self.__providers[name] = provider

    def remove_provider(self, name):
        if name in self.__providers:
            self.__providers.pop(name)

    def init_provider(self, provider, config, user_id):
        pass

    def get_recommendations(self, user_id):
        """
        Algo:
        Add all providers to threads.
        Wait for x seconds
        return results
        """
        for name, provider in self.__providers.items():
            args = self.__algo_spec[name]["config"]
            args["user_id"] = user_id
            worker = Thread(
                target=lambda q, config: q.put(provider.get_items(config=config)),
                args=(que, args),
            )
            worker.start()
            threads_list.append(worker)

        # Join all the threads
        time.sleep(1)
        for t in threads_list:
            t.join()

        # Check thread's return value
        results = []
        while not que.empty():
            result = que.get()
            results.append(result)

        return results



from typing import Optional

from yodu.provider.provider_base import ProviderBase
from yodu.recommeder.db.influx_db import InfluxDb


class TopItemsByUserAction(ProviderBase):
    action_type: Optional[str]
    tag: Optional[str]
    user_id: Optional[str]
    next_token: Optional[str]
    __influx_db_client = InfluxDb().get_client()

    def init(self, kwargs):
        if "action_type" in kwargs:
            self.action_type = kwargs["action_type"]
        if "tag" in kwargs:
            self.tag = kwargs["tag"]
        if "User_id" in kwargs:
            self.user_id = kwargs["user_id"]
        if "next_token" in kwargs:
            self.next_token = kwargs["next_token"]

    def get_items(self, config):
        """
        - Get action sorted by "tag" type
        - Get items wich same "tag" type
        :param action_type:
        :param user_id:
        :return:
        """
        self.init(config)

        print(self.action_type)
        print(self.tag)
        print(self.user_id)
        print(self.next_token)
        return {}



class ESClient:
    def __int__(self):
        pass

    def add_records(self, items):
        return items

    def get_records(self):
        pass

    def add_users(self, users):
        return users

    def add_actions(self, actions):
        return actions

    def query(self, **kwargs):
        pass



from datetime import datetime
from typing import Optional, Dict
from uuid import uuid4, UUID

from pydantic import BaseModel, Field


class Action(BaseModel):
    id_: str = Field(default_factory=uuid4)
    item_id: str
    user_id: str
    type: str
    value: float
    tags: Optional[Dict]
    created_at: datetime = Field(default_factory=datetime.now)



__title__ = "Yodu.ai Recommendation Engine"
__version__ = "0.0.1"
__author__ = "Shashank Agarwal"
__license__ = "Apache"
__copyright__ = "Copyright 2022-present The Geek Labs"

# Synonyms
TITLE = __title__
VERSION = __version__
AUTHOR = __author__
LICENSE = __license__
COPYRIGHT = __copyright__
import yodu.settings



import os

from dotenv import load_dotenv

load_dotenv()

INFLUX_DB_HOST = os.getenv("INFLUX_DB_HOST", "localhost")
INFLUX_DB_USER = os.getenv("INFLUX_DB_USER", "admin")
INFLUX_DB_PASSWORD = os.getenv("INFLUX_DB_PASSWORD", "password")
INFLUX_DB_PORT = os.getenv("INFLUX_DB_PORT", 8086)
INFLUX_USE_SSL = os.getenv("INFLUX_USE_SSL", False)
INFLUX_USE_VERIFY_SSL = os.getenv("INFLUX_USE_VERIFY_SSL", False)
INFLUX_DB_BUCKET = os.getenv("INFLUX_DB_BUCKET", "bridgeml")
INFLUX_DB_ORG = os.getenv("INFLUX_DB_ORG", "influxdata")
